@@ -1,3 +1,4 @@
+from queue import Queue
 import warnings
 from collections import Sequence
 
@@ -88,7 +89,6 @@ lg.add('RPAREN', r'\)')
 lg.add('PERCENT', r'%')
 lg.add('COMMA', r',')
 lg.add('THINARROW', r'->')
-lg.add('COLONCOLON', r'::')
 lg.add('COLON', r':')
 lg.add('CALET', r'\^')
 lg.add('OPPLUS', r'\+')
@@ -454,6 +454,7 @@ def binding(p):
 @pg.production('expr : for_expr')
 @pg.production('expr : block_expr')
 @pg.production('expr : dot_expr')
+@pg.production('expr : get_expr')
 @pg.production('expr : send_msg_expr')
 @pg.production('expr : quote_expr')
 @pg.production('expr : quasi_quote_expr')
@@ -619,6 +620,65 @@ def dot_expr(p):
     return [Symbol('getattr'), p[0], p[1].getstr()[1:]]
 
 
+@pg.production('get_expr : binop_expr LBRACK binop_expr RBRACK')
+def get_expr(p):
+    return [Symbol('get'), p[0], p[2]]
+
+
+@pg.production('get_expr : binop_expr LBRACK range_start COLON range_end RBRACK')
+def get_slice_expr(p):
+    return [Symbol('get'), p[0], p[2], p[4]]
+
+
+@pg.production('get_expr : binop_expr LBRACK range_start COLON range_end COLON range_interval RBRACK')
+def get_slice_expr(p):
+    return [Symbol('get'), p[0], p[2], p[4], p[6]]
+
+
+@pg.production('range_start : ')
+@pg.production('range_end : ')
+@pg.production('range_interval : ')
+def range_start_none(p):
+    return Symbol('None')
+
+
+@pg.production('range_start : binop_expr')
+@pg.production('range_end : binop_expr')
+@pg.production('range_interval : binop_expr')
+def range_start_none(p):
+    return p[0]
+
+
+#@pg.production('get_expr : binop_expr LBRACK binop_expr COLON binop_expr RBRACK')
+#def get_slice_expr(p):
+#    return [Symbol('get'), p[0], p[2], p[4]]
+
+
+#@pg.production('get_expr : binop_expr LBRACK binop_expr COLON RBRACK')
+#def get_slice_expr(p):
+#    return [Symbol('get'), p[0], p[2], Symbol('None')]
+
+
+#@pg.production('get_expr : binop_expr LBRACK COLON binop_expr RBRACK')
+#def get_slice_expr(p):
+#    return [Symbol('get'), p[0], Symbol('None'), p[4]]
+
+
+#@pg.production('get_expr : binop_expr LBRACK COLON RBRACK')
+#def get_slice_expr(p):
+#    return [Symbol('get'), p[0], Symbol('None'), Symbol('None')]
+
+
+#@pg.production('get_expr : binop_expr LBRACK binop_expr COLON binop_expr COLON binop_expr RBRACK')
+#def get_slice_expr(p):
+#    return [Symbol('get'), p[0], p[2], p[4], p[6]]
+
+
+#@pg.production('get_expr : binop_expr LBRACK binop_expr COLON binop_expr COLON RBRACK')
+#def get_slice_expr(p):
+#    return [Symbol('get'), p[0], p[2], p[4], p[6]]
+
+
 @pg.production('send_msg_expr : expr BANG expr')
 def dot_expr(p):
     return [Symbol('send'), p[2], p[0]]
@@ -814,11 +874,6 @@ def app_expr(p):
     return [p[0]] + p[1]
 
 
-@pg.production('app_expr : expr app_args')
-def app_expr(p):
-    return [p[0]] + p[1]
-
-
 @pg.production('app_args : LPAREN app_args_elts RPAREN')
 def app_args(p):
     return p[1]
@@ -844,11 +899,22 @@ def app_args_elt(p):
     return [token_to_keyword(p[0]), p[2]]
 
 
+@pg.production('app_args_elt : EQUALS NAME')
+def app_args_elt_short(p):
+    return [token_to_keyword(p[1]), token_to_symbol(p[1])]
+
+
 @pg.production('app_args_elt : binop_expr')
 def app_args_elt(p):
     return [p[0]]
 
 
+@pg.production('app_expr : expr app_args fn_expr')
+@pg.production('app_expr : expr app_args block_expr')
+def trailing_closure_expr(p):
+    return [p[0]] + p[1] + [p[2]]
+
+
 @pg.production('app_nc_expr : expr app_nc_args')
 def app_expr(p):
     return [p[0]] + p[1]
@@ -920,6 +986,12 @@ def field(p):
     return [p[0], p[2]]
 
 
+@pg.production('field : EQUALS NAME')
+def field(p):
+    s = token_to_symbol(p[1])
+    return [s.name, s]
+
+
 @pg.production('key : binop_expr')
 def key(p):
     return p[0]
@@ -1139,7 +1211,7 @@ def mod_lex(lexer, repl_mode=False):
     paren_openers = {'LPAREN', 'LBRACE', 'LBRACK'}
     paren_closers = {'RPAREN', 'RBRACE', 'RBRACK'}
 
-    token_queue = []
+    token_queue = Queue()
     indent_level = [0]
     ignore_newline = False
     paren_level = 0
@@ -1153,17 +1225,17 @@ def mod_lex(lexer, repl_mode=False):
             indent_level.append(indent)
             indent_token = Token('INDENT', indent_str)
             indent_token.source_pos = token.getsourcepos()
-            token_queue.append(indent_token)
+            token_queue.put(indent_token)
         else:
             while indent < indent_level[-1]:
                 indent_level.pop()
                 dedent_token = Token('DEDENT', indent_str)
-                token_queue.append(dedent_token)
+                token_queue.put(dedent_token)
         return token
 
     for token in lexer:
-        while len(token_queue) > 0:
-            queued_token = token_queue.pop()
+        while not token_queue.empty():
+            queued_token = token_queue.get()
             if queued_token.gettokentype() in paren_openers:
                 paren_level += 1
             elif queued_token.gettokentype() in paren_closers:
@@ -1209,8 +1281,8 @@ def mod_lex(lexer, repl_mode=False):
             indent_level.pop()
             yield Token('DEDENT', '')
 
-        for token in token_queue:
-            yield token
+        while not token_queue.empty():
+            yield token_queue.get()
 
 
 def lex(input, repl_mode=False):
